package main

import "xyess"
import "fmt"
import "net/http"
import "os"
import "io"
import "flag"
import "sync"
import "path"
import "time"
import "net/url"

var httpClient = &http.Client{}

type arrayFlags []string

func ( i *arrayFlags)String()string{
    return ""
}

func ( i *arrayFlags )Set(value string)error{
    *i = append( *i, value )
    return nil
}

var url_list arrayFlags
var enable_log bool
var download_timeout int
var download_count int
var ak,sk,fpath,xrang string

func init(){
    flag.IntVar( &download_timeout, "timeout", -1, "download timeout/seconds" )
    flag.IntVar( &download_count, "count", 1, "download count" )
    flag.Var( &url_list, "url", "url list" )
    flag.BoolVar(&enable_log, "log", false, "turn on log")
    flag.StringVar(&ak, "ak", "", "Access Key ID")
    flag.StringVar(&sk, "sk", "", "Secret Access Key")
    flag.StringVar(&fpath, "savepath", "./", "save local path")
    flag.StringVar(&xrang, "range", "", "download range")
}

func download_file( manager *xyess.GlobalDownManager, uri string )error{

    task, err := manager.NewDownloadTask( uri )
    if err != nil{
	return err
    }

    defer task.Stop()

    objurl, err := url.Parse( task.GetDownloadURL() )
    if err != nil{
	return err
    }

    fd, err := os.Create( path.Join( fpath, path.Base(objurl.Path) ) )
    if err != nil{
        return err
    }

    defer fd.Close()

    req, err := http.NewRequest("GET", task.GetDownloadURL(), nil )
    if err != nil{
        return err
    }

    if xrang != ""{
        req.Header.Add("Range", xrang)
    }

    resp, err := httpClient.Do(req)
    if err != nil{
        return err
    }

    defer resp.Body.Close()

    if resp.StatusCode != http.StatusOK && resp.StatusCode != http.StatusPartialContent {
        return fmt.Errorf("bad status:%v", resp.Status)
    }

    //go func(){
    //    for{
    //            fmt.Println( "task Info:", task.GetTaskInfo() )
    //            time.Sleep( 5*time.Second )
    //    }
    //}()

    if _, err := io.Copy( fd, resp.Body ); err != nil{
    	return err
    }

    return nil
}

func main(){

    var wg sync.WaitGroup

    flag.Parse()

    if len(url_list) <= 0 {
        fmt.Println( "url is empty!!!" )
        return
    }

    if download_timeout >= 0{
    	httpClient.Timeout = time.Second * time.Duration(download_timeout)
    }

    download_manager, err := xyess.NewGlobalDownManager( ak, sk )
    if err != nil{
        fmt.Println( err )
        return
    }
	
    download_manager.EnableLog( enable_log )

    for index := range url_list{
        wg.Add(1)
        go func( url string ){
            defer wg.Done()
	    for i:=0;i<download_count;i++{
            	if err := download_file( download_manager, url ); err != nil{
		    fmt.Printf( "libxyess - %s [%d] fails.errMsg:%v\n", url, i, err )
		    continue
		}
		fmt.Printf( "libxyess - %s [%d] success.\n", url, i )
	    }
        }( url_list[index] )
    }

    wg.Wait()
}

