package xyess

import "fmt"
import "strings"
import "sync/atomic"
import "xyess/bridge"

const(
    XY_OK = 0
    XY_ERR = -1
    XY_FIN = -2
    XY_TIMEOUT = -3
)

type DownloadTask struct{
    origin_url string
    rewrite_url string
}

func NewDownloadTask( origin_url, rewrite_url string )*DownloadTask{

    return &DownloadTask{origin_url:origin_url,rewrite_url:rewrite_url}
}

func ( obj *DownloadTask )GetTaskInfo()string{
    return bridge.XY_SDK_GET_INFO( obj.rewrite_url )
}

func ( obj *DownloadTask )Stop(){
    bridge.XY_STOP_TASK( obj.rewrite_url )
}

func ( obj *DownloadTask )GetDownloadURL()string{
    return obj.rewrite_url
}

type GlobalDownManager struct{
    lock int32
}

var g_GlobalDownManger GlobalDownManager

func ( obj *GlobalDownManager )init( ak string, sk string )( error ){

    if atomic.SwapInt32( &obj.lock, 1 ) != 0 {
        return nil
    }

    if ak == "" || sk == "" {
        return fmt.Errorf( "invalid ak or sk" )
    }

    if errnum := bridge.XY_SDK_INIT( ak, sk ); errnum != XY_OK {
        return fmt.Errorf( "sdk init fails. errnum:%d", errnum )
    }

    return nil
}

func ( obj *GlobalDownManager )NewDownloadTask( origin_url string )( *DownloadTask, error){

    if !strings.HasPrefix( origin_url, "http://" ){
        origin_url = "http://" + origin_url
    }

    errnum, rewrite_url := bridge.XY_SDK_REWRITE( origin_url )
    if errnum != 0 {
        return nil, fmt.Errorf( "sdk rewrite url fails. errnum:%d", errnum )
    }

    return NewDownloadTask(origin_url, rewrite_url), nil
}

func ( obj *GlobalDownManager )GetVersion()string{
    return bridge.XY_SDK_VERSION()
}

func ( obj *GlobalDownManager )EnableLog( enable bool ){
    bridge.XY_SDK_LOG( enable )
}

func ( obj *GlobalDownManager )Release(){

    if atomic.SwapInt32( &obj.lock, 0 ) != 1 {
        return
    }

    bridge.XY_SDK_RELEASE()
}

func NewGlobalDownManager( ak string, sk string )( *GlobalDownManager, error ){

    obj := &g_GlobalDownManger

    if err := obj.init( ak, sk ); err != nil{
        return nil, err
    }

    return obj, nil
}


